@@ -2,6 +2,7 @@
 # vim:fileencoding=utf-8
 # License: GPLv3 Copyright: 2020, Kovid Goyal <kovid at kovidgoyal.net>
 
+import glob
 import io
 import os
 import shlex
@@ -9,6 +10,7 @@ import shutil
 import subprocess
 import sys
 import tarfile
+import time
 from urllib.request import urlopen
 
 BUNDLE_URL = 'https://download.calibre-ebook.com/ci/kitty/{}-64.tar.xz'
@@ -17,14 +19,44 @@ is_macos = 'darwin' in sys.platform.lower()
 SW = None
 
 
-def run(*a):
+def do_print_crash_reports():
+    print('Printing available crash reports...')
+    if is_macos:
+        end_time = time.monotonic() + 90
+        while time.monotonic() < end_time:
+            time.sleep(1)
+            items = glob.glob(os.path.join(os.path.expanduser('~/Library/Logs/DiagnosticReports'), 'kitty-*.ips'))
+            if items:
+                break
+        if items:
+            time.sleep(1)
+            print(os.path.basename(items[0]))
+            sdir = os.path.dirname(os.path.abspath(__file__))
+            subprocess.check_call([sys.executable, os.path.join(sdir, 'macos_crash_report.py'), items[0]])
+    else:
+        run('sh -c "echo bt | coredumpctl debug"')
+    print(flush=True)
+
+
+def run(*a, print_crash_reports=False):
     if len(a) == 1:
         a = shlex.split(a[0])
-    print(' '.join(map(shlex.quote, a)))
+    cmd = ' '.join(map(shlex.quote, a))
+    print(cmd)
     sys.stdout.flush()
     ret = subprocess.Popen(a).wait()
     if ret != 0:
-        raise SystemExit(ret)
+        if ret < 0:
+            import signal
+            try:
+                sig = signal.Signals(-ret)
+            except ValueError:
+                pass
+            else:
+                if print_crash_reports:
+                    do_print_crash_reports()
+                raise SystemExit(f'The following process was killed by signal: {sig.name}:\n{cmd}')
+        raise SystemExit(f'The following process failed with exit code: {ret}:\n{cmd}')
 
 
 def install_deps():
@@ -43,7 +75,7 @@ def install_deps():
         run('sudo apt-get install -y libgl1-mesa-dev libxi-dev libxrandr-dev libxinerama-dev ca-certificates'
             ' libxcursor-dev libxcb-xkb-dev libdbus-1-dev libxkbcommon-dev libharfbuzz-dev libx11-xcb-dev zsh'
             ' libpng-dev liblcms2-dev libfontconfig-dev libxkbcommon-x11-dev libcanberra-dev libxxhash-dev uuid-dev'
-            ' zsh bash dash')
+            ' libsimde-dev libsystemd-dev zsh bash dash systemd-coredump gdb')
         # for some reason these directories are world writable which causes zsh
         # compinit to break
         run('sudo chmod -R og-w /usr/share/zsh')
@@ -59,13 +91,18 @@ def install_deps():
 def build_kitty():
     python = shutil.which('python3') if is_bundle else sys.executable
     cmd = f'{python} setup.py build --verbose'
+    if is_macos:
+        cmd += ' --debug'  # for better crash report to debug SIGILL issue
     if os.environ.get('KITTY_SANITIZE') == '1':
         cmd += ' --debug --sanitize'
     run(cmd)
 
 
 def test_kitty():
-    run('./test.py')
+    if is_macos:
+        run('ulimit -c unlimited')
+        run('sudo chmod -R 777 /cores')
+    run('./test.py', print_crash_reports=True)
 
 
 def package_kitty():
