@@ -292,8 +292,8 @@ class Freeze(object):
                 'harfbuzz.0',
                 'png16.16',
                 'lcms2.2',
-                'crypto.1.1',
-                'ssl.1.1',
+                'crypto.3',
+                'ssl.3',
                 'xxhash.0',
         ):
             print('\nAdding', x)
