@@ -14,6 +14,6 @@ typedef unsigned long long notification_id_type;
 typedef void (*GLFWDBusnotificationcreatedfun)(notification_id_type, uint32_t, void*);
 typedef void (*GLFWDBusnotificationactivatedfun)(uint32_t, const char*);
 notification_id_type
-glfw_dbus_send_user_notification(const char *app_name, const char* icon, const char *summary, const char *body, const char *action_name, int32_t timeout, GLFWDBusnotificationcreatedfun, void*);
+glfw_dbus_send_user_notification(const char *app_name, const char* icon, const char *summary, const char *body, const char *action_name, int32_t timeout, int urgency, GLFWDBusnotificationcreatedfun, void*);
 void
 glfw_dbus_set_user_notification_activated_handler(GLFWDBusnotificationactivatedfun handler);
