@@ -847,7 +847,7 @@ def parse_cmdline(oc: Options, disabled: OptionSpecSeq, ans: Any, args: Optional
 def options_spec() -> str:
     if not hasattr(options_spec, 'ans'):
         OPTIONS = '''
---class
+--class --app-id
 dest=cls
 default={appname}
 condition=not is_macos
@@ -896,11 +896,13 @@ use :code:`open -a kitty.app -n` instead.
 --session
 completion=type:file ext:session relative:conf group:"Session files"
 Path to a file containing the startup :italic:`session` (tabs, windows, layout,
-programs). Use - to read from STDIN. See the :file:`README` file for details and
+programs). Use - to read from STDIN. See :ref:`sessions` for details and
 an example. Environment variables in the file name are expanded,
 relative paths are resolved relative to the kitty configuration directory.
 The special value :code:`none` means no session will be used, even if
 the :opt:`startup_session` option has been specified in kitty.conf.
+Note that using this option means the command line arguments to kitty specifying
+a program to run are ignored.
 
 
 --hold
