@@ -9,6 +9,7 @@
 #include "state.h"
 #include "cleanup.h"
 #include "monotonic.h"
+#include <Availability.h>
 #include <Carbon/Carbon.h>
 #include <Cocoa/Cocoa.h>
 #ifndef KITTY_USE_DEPRECATED_MACOS_NOTIFICATION_API
@@ -391,8 +392,8 @@ set_notification_activated_callback(PyObject *self UNUSED, PyObject *callback) {
 
 static PyObject*
 cocoa_send_notification(PyObject *self UNUSED, PyObject *args) {
-    char *identifier = NULL, *title = NULL, *informativeText = NULL, *subtitle = NULL;
-    if (!PyArg_ParseTuple(args, "zsz|z", &identifier, &title, &informativeText, &subtitle)) return NULL;
+    char *identifier = NULL, *title = NULL, *informativeText = NULL, *subtitle = NULL; int urgency = 1;
+    if (!PyArg_ParseTuple(args, "zsz|zi", &identifier, &title, &informativeText, &subtitle, &urgency)) return NULL;
     NSUserNotificationCenter *center = [NSUserNotificationCenter defaultUserNotificationCenter];
     if (!center) {PyErr_SetString(PyExc_RuntimeError, "Failed to get the user notification center"); return NULL; }
     if (!center.delegate) center.delegate = [[NotificationDelegate alloc] init];
@@ -438,10 +439,25 @@ cocoa_send_notification(PyObject *self UNUSED, PyObject *args) {
     }
 @end
 
+static UNUserNotificationCenter*
+get_notification_center_safely(void) {
+    NSBundle *b = [NSBundle mainBundle];
+    // when bundleIdentifier is nil currentNotificationCenter crashes instead
+    // of returning nil. Apple...purveyor of shiny TOYS
+    if (!b || !b.bundleIdentifier) return nil;
+    UNUserNotificationCenter *center = nil;
+    @try {
+        center = [UNUserNotificationCenter currentNotificationCenter];
+    } @catch (NSException *e) {
+        log_error("Failed to get current UNUserNotificationCenter object with error: %s (%s)",
+                            [[e name] UTF8String], [[e reason] UTF8String]);
+    }
+    return center;
+}
 
 static void
-schedule_notification(const char *identifier, const char *title, const char *body, const char *subtitle) {
-    UNUserNotificationCenter *center = [UNUserNotificationCenter currentNotificationCenter];
+schedule_notification(const char *identifier, const char *title, const char *body, const char *subtitle, int urgency) {
+    UNUserNotificationCenter *center = get_notification_center_safely();
     if (!center) return;
     // Configure the notification's payload.
     UNMutableNotificationContent* content = [[UNMutableNotificationContent alloc] init];
@@ -449,6 +465,22 @@ schedule_notification(const char *identifier, const char *title, const char *bod
     if (body) content.body = @(body);
     if (subtitle) content.subtitle = @(subtitle);
     content.sound = [UNNotificationSound defaultSound];
+#if __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000
+    switch (urgency) {
+        case 0:
+            content.interruptionLevel = UNNotificationInterruptionLevelPassive;
+        case 2:
+            content.interruptionLevel = UNNotificationInterruptionLevelCritical;
+        default:
+            content.interruptionLevel = UNNotificationInterruptionLevelActive;
+    }
+#else
+    if ([content respondsToSelector:@selector(interruptionLevel)]) {
+        NSUInteger level = 1;
+        if (urgency == 0) level = 0; else if (urgency == 2) level = 3;
+        [content setValue:@(level) forKey:@"interruptionLevel"];
+    }
+#endif
     // Deliver the notification
     static unsigned long counter = 1;
     UNNotificationRequest* request = [
@@ -465,6 +497,7 @@ schedule_notification(const char *identifier, const char *title, const char *bod
 
 typedef struct {
     char *identifier, *title, *body, *subtitle;
+    int urgency;
 } QueuedNotification;
 
 typedef struct {
@@ -474,13 +507,14 @@ typedef struct {
 static NotificationQueue notification_queue = {0};
 
 static void
-queue_notification(const char *identifier, const char *title, const char* body, const char* subtitle) {
+queue_notification(const char *identifier, const char *title, const char* body, const char* subtitle, int urgency) {
     ensure_space_for((&notification_queue), notifications, QueuedNotification, notification_queue.count + 16, capacity, 16, true);
     QueuedNotification *n = notification_queue.notifications + notification_queue.count++;
     n->identifier = identifier ? strdup(identifier) : NULL;
     n->title = title ? strdup(title) : NULL;
     n->body = body ? strdup(body) : NULL;
     n->subtitle = subtitle ? strdup(subtitle) : NULL;
+    n->urgency = urgency;
 }
 
 static void
@@ -488,25 +522,25 @@ drain_pending_notifications(BOOL granted) {
     if (granted) {
         for (size_t i = 0; i < notification_queue.count; i++) {
             QueuedNotification *n = notification_queue.notifications + i;
-            schedule_notification(n->identifier, n->title, n->body, n->subtitle);
+            schedule_notification(n->identifier, n->title, n->body, n->subtitle, n->urgency);
         }
     }
     while(notification_queue.count) {
         QueuedNotification *n = notification_queue.notifications + --notification_queue.count;
         free(n->identifier); free(n->title); free(n->body); free(n->subtitle);
-        n->identifier = NULL; n->title = NULL; n->body = NULL; n->subtitle = NULL;
+        memset(n, 0, sizeof(QueuedNotification));
     }
 }
 
 static PyObject*
 cocoa_send_notification(PyObject *self UNUSED, PyObject *args) {
-    char *identifier = NULL, *title = NULL, *body = NULL, *subtitle = NULL;
-    if (!PyArg_ParseTuple(args, "zsz|z", &identifier, &title, &body, &subtitle)) return NULL;
+    char *identifier = NULL, *title = NULL, *body = NULL, *subtitle = NULL; int urgency = 1;
+    if (!PyArg_ParseTuple(args, "zsz|zi", &identifier, &title, &body, &subtitle, &urgency)) return NULL;
 
-    UNUserNotificationCenter *center = [UNUserNotificationCenter currentNotificationCenter];
+    UNUserNotificationCenter *center = get_notification_center_safely();
     if (!center) Py_RETURN_NONE;
     if (!center.delegate) center.delegate = [[NotificationDelegate alloc] init];
-    queue_notification(identifier, title, body, subtitle);
+    queue_notification(identifier, title, body, subtitle, urgency);
 
     // The badge permission needs to be requested as well, even though it is not used,
     // otherwise macOS refuses to show the preference checkbox for enable/disable notification sound.
@@ -736,12 +770,14 @@ cocoa_create_global_menu(void) {
 
 void
 cocoa_update_menu_bar_title(PyObject *pytitle) {
+    if (!pytitle) return;
     NSString *title = nil;
     if (OPT(macos_menubar_title_max_length) > 0 && PyUnicode_GetLength(pytitle) > OPT(macos_menubar_title_max_length)) {
         static char fmt[64];
         snprintf(fmt, sizeof(fmt), "%%%ld.%ldU%%s", OPT(macos_menubar_title_max_length), OPT(macos_menubar_title_max_length));
         RAII_PyObject(st, PyUnicode_FromFormat(fmt, pytitle, "…"));
         if (st) title = @(PyUnicode_AsUTF8(st));
+        else PyErr_Print();
     } else {
         title = @(PyUnicode_AsUTF8(pytitle));
     }
