@@ -7,10 +7,7 @@ import textwrap
 import typing
 from functools import lru_cache
 from importlib import import_module
-from typing import (
-    Any, Callable, Dict, Iterable, Iterator, List, Match, Optional, Set, Tuple,
-    Union, cast
-)
+from typing import Any, Callable, Dict, Iterable, Iterator, List, Match, Optional, Set, Tuple, Union, cast
 
 import kitty.conf.utils as generic_parsers
 from kitty.constants import website_url
@@ -45,41 +42,87 @@ def expand_opt_references(conf_name: str, text: str) -> str:
 
 
 @run_once
-def ref_map() -> Dict[str, str]:
-    from kitty.actions import get_all_actions
-    ref_map = {
-        'layouts': f'{website_url("overview")}#layouts',
-        'watchers': f'{website_url("launch")}#watchers',
-        'sessions': f'{website_url("overview")}#startup-sessions',
-        'functional': f'{website_url("keyboard-protocol")}#functional-key-definitions',
-        'shell_integration': website_url("shell-integration"),
-        'github_discussions': 'https://github.com/kovidgoyal/kitty/discussions',
-    }
-    for actions in get_all_actions().values():
-        for ac in actions:
-            ref_map[f'action-{ac.name}'] = f'{website_url("actions")}#' + ac.name.replace('_', '-')
-    return ref_map
+def ref_map() -> Dict[str, Dict[str, str]]:
+    import json
+
+    from ..fast_data_types import get_docs_ref_map
+    ans: Dict[str, Dict[str, str]] = json.loads(get_docs_ref_map())
+    return ans
+
+
+def resolve_ref(ref: str, website_url: Callable[[str], str] = website_url) -> str:
+    m = ref_map()
+    href = m['ref'].get(ref, '')
+    prefix, rest = ref.partition('-')[::2]
+    if href:
+        pass
+    elif ref.startswith('conf-kitty-'):
+        href = f'conf#{ref}'
+    elif ref.startswith('conf-kitten-'):
+        parts = ref.split('-')
+        href = "kittens/" + parts[2] + f'/#{ref}'
+    elif ref.startswith('at_'):
+        base = ref.split('_', 1)[1]
+        href = "remote-control/#at-" + base.replace('_', '-')
+    elif ref.startswith('at-'):
+        base = ref.split('-', 1)[1]
+        href = "remote-control/#at-" + base.replace('_', '-')
+    elif ref.startswith('action-group-'):
+        href = f'actions/#{ref}'
+    elif prefix == 'action':
+        href = f'actions/#{rest.replace("_", "-")}'
+    elif prefix in ('term', 'envvar'):
+        href = 'glossary/#' + ref
+    elif prefix == 'doc':
+        href = rest.lstrip('/')
+    elif prefix in ('issues', 'pull', 'discussions'):
+        t, num = ref.partition(':')[::2]
+        href = f'https://github.com/kovidgoyal/kitty/{prefix}/{rest}'
+    if not (href.startswith('https://') or href.startswith('http://')):
+        href = website_url(href)
+    return href
 
 
 def remove_markup(text: str) -> str:
 
+    imap = {'iss': 'issues-', 'pull': 'pull-', 'disc': 'discussions-'}
+
+    def extract(m: 'Match[str]') -> Tuple[str, str]:
+        parts = m.group(2).split('<')
+        t = parts[0].strip()
+        q = parts[-1].rstrip('>')
+        return t, q
+
     def sub(m: 'Match[str]') -> str:
-        if m.group(1) == 'ref':
-            q = m.group(2).split('<')[-1].rstrip('>')
-            return ref_map()[q]
-        if m.group(1) == 'ac':
-            q = m.group(2).split('<')[-1].rstrip('>')
-            return ref_map()[f'action-{q}']
-        if m.group(1) == 'disc':
-            parts = m.group(2).split('<')
-            t = parts[0].strip()
-            q = parts[-1].rstrip('>')
-            return f'{t} {ref_map()["github_discussions"]}/{q}'
+        key = m.group(1)
+        if key in ('ref', 'iss', 'pull', 'disc'):
+            t, q = extract(m)
+            q = imap.get(key, '') + q
+            url = resolve_ref(q)
+            if not url:
+                raise KeyError(f'Failed to resolve :{m.group(1)}: {q}')
+            return f'{t} <{url}>'
+        if key == 'doc':
+            t, q = extract(m)
+            return f'{t} <{website_url(q)}>'
+        if key in ('term', 'option'):
+            t, _ = extract(m)
+            return t
+        if key in ('ac', 'opt'):
+            t, q = extract(m)
+            return f'{t} {q}' if q and q != t else t
+        if key == 'code':
+            return m.group(2).replace('\\\\', '\\')
+
         return str(m.group(2))
 
     return re.sub(r':([a-zA-Z0-9]+):`(.+?)`', sub, text, flags=re.DOTALL)
 
 
+def strip_inline_literal(text: str) -> str:
+    return re.sub(r'``([^`]+)``', r'`\1`', text, flags=re.DOTALL)
+
+
 def iter_blocks(lines: Iterable[str]) -> Iterator[Tuple[List[str], int]]:
     current_block: List[str] = []
     prev_indent = 0
@@ -121,6 +164,7 @@ def wrapped_block(lines: Iterable[str], comment_symbol: str = '#: ') -> Iterator
 
 def render_block(text: str, comment_symbol: str = '#: ') -> str:
     text = remove_markup(text)
+    text = strip_inline_literal(text)
     lines = text.splitlines()
     return '\n'.join(wrapped_block(lines, comment_symbol))
 
@@ -224,11 +268,12 @@ class Option:
             return ans
         mopts = [self] + option_group
         a('.. opt:: ' + ', '.join(f'{conf_name}.{mo.name}' for mo in mopts))
-        a('.. code-block:: conf')
-        a('')
-        sz = max(len(x.name) for x in mopts)
-        for mo in mopts:
-            a(('    {:%ds} {}' % sz).format(mo.name, mo.defval_as_string))
+        if any(mo.defval_as_string for mo in mopts):
+            a('.. code-block:: conf')
+            a('')
+            sz = max(len(x.name) for x in mopts)
+            for mo in mopts:
+                a(('    {:%ds} {}' % sz).format(mo.name, mo.defval_as_string))
         a('')
         if self.long_text:
             a(expand_opt_references(conf_name, self.long_text))
@@ -268,7 +313,11 @@ class MultiOption:
         for k in self.items:
             if k.documented:
                 documented = True
-                a(f'{self.name} {k.defval_as_str if k.add_to_default else ""}'.rstrip())
+                if k.add_to_default:
+                    a(f'{self.name} {k.defval_as_str}'.rstrip())
+                else:
+                    # Comment out multi-options that have no default values
+                    a(f'# {self.name}'.rstrip())
                 if not k.add_to_default and k.defval_as_str:
                     a('')
                     a(f'#: E.g. {self.name} {k.defval_as_str}'.rstrip())
@@ -282,11 +331,22 @@ class MultiOption:
         ans: List[str] = []
         a = ans.append
         a(f'.. opt:: {conf_name}.{self.name}')
-        a('.. code-block:: conf')
-        a('')
-        for k in self.items:
-            if k.documented:
-                a(f'    {self.name:s} {k.defval_as_str}'.rstrip())
+        documented = tuple(x for x in self.items if x.documented)
+        if any(k.defval_as_str for k in documented):
+            defaults = tuple(x for x in documented if x.add_to_default)
+            if defaults:
+                a('.. code-block:: conf')
+                a('')
+                for k in defaults:
+                    a(f'    {self.name:s} {k.defval_as_str}'.rstrip())
+            else:
+                a('')
+                a('Has no default values. Example values are shown below:')
+                a('')
+                a('.. code-block:: conf')
+                a('')
+                for k in self.items:
+                    a(f'    {self.name:s} {k.defval_as_str}'.rstrip())
         a('')
         if self.long_text:
             a(expand_opt_references(conf_name, self.long_text))
@@ -544,6 +604,9 @@ class Group:
 
             if commented:
                 ans = [x if x.startswith('#') or not x.strip() else (f'# {x}') for x in ans]
+            else:
+                # Comment out any invalid options that have no value
+                ans = [f'# {x}' if not x.startswith('#') and len(x.strip().split()) == 1 else x for x in ans]
 
         return ans
 
