@@ -21,7 +21,7 @@
 #import <Foundation/NSString.h>
 #import <Foundation/NSDictionary.h>
 
-#define debug(...) if (global_state.debug_rendering) { fprintf(stderr, __VA_ARGS__); fflush(stderr); }
+#define debug debug_fonts
 
 typedef struct {
     PyObject_HEAD
@@ -184,38 +184,79 @@ glyph_id_for_codepoint_ctfont(CTFontRef ct_font, char_type ch) {
     return glyphs[0];
 }
 
+static bool
+cf_string_equals(CFStringRef a, CFStringRef b) { return CFStringCompare(a, b, 0) == kCFCompareEqualTo; }
+
+#define LAST_RESORT_FONT_NAME "LastResort"
+
 static bool
 is_last_resort_font(CTFontRef new_font) {
     CFStringRef name = CTFontCopyPostScriptName(new_font);
-    CFComparisonResult cr = CFStringCompare(name, CFSTR("LastResort"), 0);
+    bool ans = cf_string_equals(name, CFSTR(LAST_RESORT_FONT_NAME));
     CFRelease(name);
-    return cr == kCFCompareEqualTo;
+    return ans;
+}
+
+static CTFontDescriptorRef _nerd_font_descriptor = NULL;
+
+static CTFontRef nerd_font(CGFloat sz) {
+    static bool searched = false;
+    if (!searched) {
+        searched = true;
+        CFArrayRef fonts = CTFontCollectionCreateMatchingFontDescriptors(all_fonts_collection());
+        const CFIndex count = CFArrayGetCount(fonts);
+        for (CFIndex i = 0; i < count; i++) {
+            CTFontDescriptorRef descriptor = (CTFontDescriptorRef)CFArrayGetValueAtIndex(fonts, i);
+            CFStringRef name = CTFontDescriptorCopyAttribute(descriptor, kCTFontNameAttribute);
+            bool is_nerd_font = cf_string_equals(name, CFSTR("SymbolsNFM"));
+            CFRelease(name);
+            if (is_nerd_font) {
+                _nerd_font_descriptor = CTFontDescriptorCreateCopyWithAttributes(descriptor, CTFontDescriptorCopyAttributes(descriptor));
+                break;
+            }
+        }
+        CFRelease(fonts);
+    }
+    return _nerd_font_descriptor ? CTFontCreateWithFontDescriptor(_nerd_font_descriptor, sz, NULL) : NULL;
+}
+
+static bool
+font_can_render_cell(CTFontRef font, CPUCell *cell) {
+    char_type ch = cell->ch ? cell->ch : ' ';
+    bool found = true;
+    if (!glyph_id_for_codepoint_ctfont(font, ch)) found = false;
+    for (unsigned i = 0; i < arraysz(cell->cc_idx) && cell->cc_idx[i] && found; i++) {
+        char_type cch = codepoint_for_mark(cell->cc_idx[i]);
+        if (!glyph_id_for_codepoint_ctfont(font, cch)) found = false;
+    }
+    return found;
 }
 
 static CTFontRef
 manually_search_fallback_fonts(CTFontRef current_font, CPUCell *cell) {
+    char_type ch = cell->ch ? cell->ch : ' ';
+    const bool in_first_pua = 0xe000 <= ch && ch <= 0xf8ff;
+    // preferentially load from NERD fonts
+    if (in_first_pua) {
+        CTFontRef nf = nerd_font(CTFontGetSize(current_font));
+        if (nf) {
+            if (font_can_render_cell(nf, cell)) return nf;
+            CFRelease(nf);
+        }
+    }
     CFArrayRef fonts = CTFontCollectionCreateMatchingFontDescriptors(all_fonts_collection());
     CTFontRef ans = NULL;
     const CFIndex count = CFArrayGetCount(fonts);
     for (CFIndex i = 0; i < count; i++) {
         CTFontDescriptorRef descriptor = (CTFontDescriptorRef)CFArrayGetValueAtIndex(fonts, i);
         CTFontRef new_font = CTFontCreateWithFontDescriptor(descriptor, CTFontGetSize(current_font), NULL);
-        if (new_font) {
-            if (!is_last_resort_font(new_font)) {
-                char_type ch = cell->ch ? cell->ch : ' ';
-                bool found = true;
-                if (!glyph_id_for_codepoint_ctfont(new_font, ch)) found = false;
-                for (unsigned i = 0; i < arraysz(cell->cc_idx) && cell->cc_idx[i] && found; i++) {
-                    ch = codepoint_for_mark(cell->cc_idx[i]);
-                    if (!glyph_id_for_codepoint_ctfont(new_font, ch)) found = false;
-                }
-                if (found) {
-                    ans = new_font;
-                    break;
-                }
+        if (!is_last_resort_font(new_font)) {
+            if (font_can_render_cell(new_font, cell)) {
+                ans = new_font;
+                break;
             }
-            CFRelease(new_font);
         }
+        CFRelease(new_font);
     }
     CFRelease(fonts);
     return ans;
@@ -231,16 +272,15 @@ find_substitute_face(CFStringRef str, CTFontRef old_font, CPUCell *cpu_cell) {
         CTFontRef new_font = CTFontCreateForString(old_font, str, CFRangeMake(start, amt));
         if (amt == len && len != 1) amt = 1;
         else start++;
-        if (new_font == NULL) { PyErr_SetString(PyExc_ValueError, "Failed to find fallback CTFont"); return NULL; }
         if (new_font == old_font) { CFRelease(new_font); continue; }
-        if (is_last_resort_font(new_font)) {
-            CFRelease(new_font);
+        if (!new_font || is_last_resort_font(new_font)) {
+            if (new_font) CFRelease(new_font);
             if (is_private_use(cpu_cell->ch)) {
                 // CoreTexts fallback font mechanism does not work for private use characters
                 new_font = manually_search_fallback_fonts(old_font, cpu_cell);
                 if (new_font) return new_font;
             }
-            PyErr_Format(PyExc_ValueError, "Failed to find fallback CTFont other than the LastResort font for: %s", [(NSString *)str UTF8String]);
+            PyErr_Format(PyExc_ValueError, "Failed to find fallback CTFont other than the %s font for: %s", LAST_RESORT_FONT_NAME, [(NSString *)str UTF8String]);
             return NULL;
         }
         return new_font;
@@ -249,8 +289,35 @@ find_substitute_face(CFStringRef str, CTFontRef old_font, CPUCell *cpu_cell) {
     return NULL;
 }
 
+static CTFontRef
+apply_styles_to_fallback_font(CTFontRef original_fallback_font, bool bold, bool italic) {
+    if (!original_fallback_font || (!bold && !italic) || is_last_resort_font(original_fallback_font)) return original_fallback_font;
+    CTFontDescriptorRef original_descriptor = CTFontCopyFontDescriptor(original_fallback_font);
+    // We cannot set kCTFontTraitMonoSpace in traits as if the original
+    // fallback font is Zapf Dingbats we get .AppleSystemUIFontMonospaced as
+    // the new fallback
+    CTFontSymbolicTraits traits = 0;
+    if (bold) traits |= kCTFontTraitBold;
+    if (italic) traits |= kCTFontTraitItalic;
+    CTFontDescriptorRef descriptor = CTFontDescriptorCreateCopyWithSymbolicTraits(original_descriptor, traits, traits);
+    CFRelease(original_descriptor);
+    if (descriptor) {
+        CTFontRef ans = CTFontCreateWithFontDescriptor(descriptor, CTFontGetSize(original_fallback_font), NULL);
+        CFRelease(descriptor);
+        if (!ans) return original_fallback_font;
+        CFStringRef new_name = CTFontCopyFamilyName(ans);
+        CFStringRef old_name = CTFontCopyFamilyName(original_fallback_font);
+        bool same_family = cf_string_equals(new_name, old_name);
+        /* NSLog(@"old: %@ new: %@", old_name, new_name); */
+        CFRelease(new_name); CFRelease(old_name);
+        if (same_family) { CFRelease(original_fallback_font); return ans; }
+        CFRelease(ans);
+    }
+    return original_fallback_font;
+}
+
 PyObject*
-create_fallback_face(PyObject *base_face, CPUCell* cell, bool UNUSED bold, bool UNUSED italic, bool emoji_presentation, FONTS_DATA_HANDLE fg) {
+create_fallback_face(PyObject *base_face, CPUCell* cell, bool bold, bool italic, bool emoji_presentation, FONTS_DATA_HANDLE fg) {
     CTFace *self = (CTFace*)base_face;
     CTFontRef new_font;
 #define search_for_fallback() \
@@ -268,21 +335,19 @@ create_fallback_face(PyObject *base_face, CPUCell* cell, bool UNUSED bold, bool
             search_for_fallback();
         }
     }
-    else { search_for_fallback(); }
+    else { search_for_fallback(); new_font = apply_styles_to_fallback_font(new_font, bold, italic); }
     if (new_font == NULL) return NULL;
-    NSURL *url = (NSURL*)CTFontCopyAttribute(new_font, kCTFontURLAttribute);
-    const char *font_path = [[url path] UTF8String];
+    PyObject *postscript_name = Py_BuildValue("s", convert_cfstring(CTFontCopyPostScriptName(new_font), true));
     ssize_t idx = -1;
     PyObject *q, *ans = NULL;
     while ((q = iter_fallback_faces(fg, &idx))) {
         CTFace *qf = (CTFace*)q;
-        const char *qpath;
-        if (qf->path && (qpath = PyUnicode_AsUTF8(qf->path)) && strcmp(qpath, font_path) == 0) {
+        if (PyObject_RichCompareBool(postscript_name, qf->postscript_name, Py_EQ) == 1) {
             ans = PyLong_FromSsize_t(idx);
             break;
         }
     }
-    [url release];
+    Py_CLEAR(postscript_name);
     if (ans == NULL) return (PyObject*)ct_face(new_font, fg);
     CFRelease(new_font);
     return ans;
@@ -449,6 +514,7 @@ finalize(void) {
     if (all_fonts_collection_data) CFRelease(all_fonts_collection_data);
     if (window_title_font) CFRelease(window_title_font);
     window_title_font = nil;
+    if (_nerd_font_descriptor) CFRelease(_nerd_font_descriptor);
 }
 
 
