@@ -16,11 +16,11 @@ from enum import Enum, auto
 from functools import partial
 from gettext import gettext as _
 from itertools import count
-from time import monotonic, time_ns
+from time import time_ns
 from typing import IO, Any, Callable, DefaultDict, Deque, Dict, Iterable, Iterator, List, Optional, Tuple, Union
 
 from kittens.transfer.utils import IdentityCompressor, ZlibCompressor, abspath, expand_home, home_path
-from kitty.fast_data_types import FILE_TRANSFER_CODE, OSC, AES256GCMDecrypt, add_timer, base64_decode, base64_encode, get_boss, get_options
+from kitty.fast_data_types import ESC_OSC, FILE_TRANSFER_CODE, AES256GCMDecrypt, add_timer, base64_decode, base64_encode, get_boss, get_options, monotonic
 from kitty.types import run_once
 
 from .utils import log_error
@@ -329,14 +329,14 @@ class FileTransmissionCommand:
     def deserialize(cls, data: Union[str, bytes, memoryview]) -> 'FileTransmissionCommand':
         ans = FileTransmissionCommand()
         fmap = serialized_to_field_map()
-        from kittens.transfer.rsync import decode_utf8_buffer, parse_ftc
+        from kittens.transfer.rsync import parse_ftc
 
         def handle_item(key: memoryview, val: memoryview) -> None:
             field = fmap.get(key)
             if field is None:
                 return
             if issubclass(field.type, Enum):
-                setattr(ans, field.name, field.type[decode_utf8_buffer(val)])
+                setattr(ans, field.name, field.type[str(val, "utf-8")])
             elif field.type is bytes:
                 setattr(ans, field.name, base64_decode(val))
             elif field.type is int:
@@ -345,7 +345,7 @@ class FileTransmissionCommand:
                 if field.metadata.get('base64'):
                     sval = base64_decode(val).decode('utf-8')
                 else:
-                    sval = safe_string(decode_utf8_buffer(val))
+                    sval = safe_string(str(val, "utf-8"))
                 setattr(ans, field.name, sval)
 
         parse_ftc(data, handle_item)
@@ -855,7 +855,7 @@ class FileTransmission:
             if self.active_sends[a].is_expired:
                 self.drop_send(a)
 
-    def handle_serialized_command(self, data: str) -> None:
+    def handle_serialized_command(self, data: memoryview) -> None:
         try:
             cmd = FileTransmissionCommand.deserialize(data)
         except Exception as e:
@@ -1147,7 +1147,7 @@ class FileTransmission:
         window = boss.window_id_map.get(self.window_id)
         if window is not None:
             data = tuple(payload.get_serialized_fields(prefix_with_osc_code=True))
-            queued = window.screen.send_escape_code_to_child(OSC, data)
+            queued = window.screen.send_escape_code_to_child(ESC_OSC, data)
             if not queued:
                 if use_pending:
                     if appendleft:
