@@ -9,4 +9,4 @@
 #include "data-types.h"
 #define uthash_fatal(msg) fatal(msg)
 #define hash_handle_type UT_hash_handle
-#include "uthash.h"
+#include "../3rdparty/uthash.h"
