@@ -69,7 +69,7 @@ def parse(lines: Iterable[str]) -> Iterator[OpenAction]:
         entries.append((tuple(match_criteria), tuple(raw_actions)))
 
     with to_cmdline_implementation.filter_env_vars(
-        'URL', 'FILE_PATH', 'FILE', 'FRAGMENT', 'URL_PATH',
+        'URL', 'FILE_PATH', 'FILE', 'FRAGMENT', 'URL_PATH', 'NETLOC',
         EDITOR=shlex.join(get_editor()),
         SHELL=resolved_shell(get_options())[0]
     ):
@@ -164,9 +164,11 @@ def actions_for_url_from_list(url: str, actions: Iterable[OpenAction]) -> Iterat
         return
     path = unquote(purl.path)
     up = purl.path
+    netloc = unquote(purl.netloc) if purl.netloc else ''
     if purl.query:
         up += f'?{purl.query}'
-    if purl.fragment:
+    frag = unquote(purl.fragment) if purl.fragment else ''
+    if frag:
         up += f'#{purl.fragment}'
 
     env = {
@@ -174,7 +176,8 @@ def actions_for_url_from_list(url: str, actions: Iterable[OpenAction]) -> Iterat
         'FILE_PATH': path,
         'URL_PATH': up,
         'FILE': posixpath.basename(path),
-        'FRAGMENT': unquote(purl.fragment)
+        'FRAGMENT': frag,
+        'NETLOC': netloc,
     }
 
     def expand(x: Any) -> Any:
@@ -230,7 +233,7 @@ def default_open_actions() -> Tuple[OpenAction, ...]:
 # Open kitty HTML docs links
 protocol kitty+doc
 action show_kitty_doc $URL_PATH
-    '''.splitlines()))
+'''.splitlines()))
 
 
 @run_once
@@ -249,26 +252,26 @@ action launch --hold --type=os-window kitty +shebang $FILE_PATH __ext__
 # Open directories
 protocol file
 mime inode/directory
-action launch --type=os-window --cwd $FILE_PATH
+action launch --type=os-window --cwd -- $FILE_PATH
 
 # Open executable file
 protocol file
 mime inode/executable,application/vnd.microsoft.portable-executable
-action launch --hold --type=os-window $FILE_PATH
+action launch --hold --type=os-window -- $FILE_PATH
 
 # Open text files without fragments in the editor
 protocol file
 mime text/*
-action launch --type=os-window $EDITOR $FILE_PATH
+action launch --type=os-window -- $EDITOR -- $FILE_PATH
 
 # Open image files with icat
 protocol file
 mime image/*
-action launch --type=os-window kitten icat --hold $FILE_PATH
+action launch --type=os-window kitten icat --hold -- $FILE_PATH
 
 # Open ssh URLs with ssh command
 protocol ssh
-action launch --type=os-window ssh $URL
+action launch --type=os-window ssh -- $URL
 '''.splitlines()))
 
 
