@@ -1,9 +1,11 @@
 #!/usr/bin/env python
 # License: GPLv3 Copyright: 2020, Kovid Goyal <kovid at kovidgoyal.net>
 
-from typing import Any, Callable, Dict, NamedTuple, Tuple
+from typing import Any, Callable, Dict, NamedTuple, Optional, Tuple, Union
 
 from .constants import is_macos, is_wayland
+from .fast_data_types import get_options
+from .options.types import Options
 from .types import FloatEdges
 from .typing import EdgeLiteral
 from .utils import log_error
@@ -35,6 +37,20 @@ def sanitize_window_size(x: Any) -> int:
     return max(20, min(ans, 50000))
 
 
+def edge_spacing(which: EdgeLiteral, opts:Optional[Union[WindowSizeData, Options]] = None) -> float:
+    if opts is None:
+        opts = get_options()
+    margin: float = getattr(opts.single_window_margin_width, which)
+    if margin < 0:
+        margin = getattr(opts.window_margin_width, which)
+
+    padding: float = getattr(opts.single_window_padding_width, which)
+    if padding < 0:
+        padding = getattr(opts.window_padding_width, which)
+    return float(padding + margin)
+
+
+
 def initial_window_size_func(opts: WindowSizeData, cached_values: Dict[str, Any]) -> Callable[[int, int, float, float, float, float], Tuple[int, int]]:
 
     if 'window-size' in cached_values and opts.remember_window_size:
