@@ -8,7 +8,7 @@ import re
 import sys
 from contextlib import suppress
 from functools import lru_cache, partial
-from time import monotonic, time_ns
+from time import time_ns
 from types import GeneratorType
 from typing import (
     TYPE_CHECKING,
@@ -34,6 +34,7 @@ from .fast_data_types import (
     EllipticCurveKey,
     get_boss,
     get_options,
+    monotonic,
     read_command_response,
     send_data_to_peer,
 )
@@ -52,12 +53,16 @@ def encode_response_for_peer(response: Any) -> bytes:
     return b'\x1bP@kitty-cmd' + json.dumps(response).encode('utf-8') + b'\x1b\\'
 
 
-def parse_cmd(serialized_cmd: str, encryption_key: EllipticCurveKey) -> Dict[str, Any]:
+def parse_cmd(serialized_cmd: memoryview, encryption_key: EllipticCurveKey) -> Dict[str, Any]:
+    # See https://github.com/python/cpython/issues/74379 for why we cant use
+    # memoryview directly :((
     try:
-        pcmd = json.loads(serialized_cmd)
+        pcmd = json.loads(bytes(serialized_cmd))
     except Exception:
+        log_error('Failed to parse JSON payload of remote command, ignoring it')
         return {}
     if not isinstance(pcmd, dict) or 'version' not in pcmd:
+        log_error('JSON payload of remote command is invalid, must be an object with a version field')
         return {}
     pcmd.pop('password', None)
     if 'encrypted' in pcmd:
@@ -271,7 +276,8 @@ they will only work if this process is run within a kitty window.
 --password
 A password to use when contacting kitty. This will cause kitty to ask the user
 for permission to perform the specified action, unless the password has been
-accepted before or is pre-configured in :file:`kitty.conf`.
+accepted before or is pre-configured in :file:`kitty.conf`. To use a blank password
+specify :option:`kitten @ --use-password` as :code:`always`.
 
 
 --password-file
@@ -295,7 +301,7 @@ default=if-available
 choices=if-available,never,always
 If no password is available, kitty will usually just send the remote control command
 without a password. This option can be used to force it to :code:`always` or :code:`never` use
-the supplied password.
+the supplied password. If set to always and no password is provided, the blank password is used.
 '''.format, appname=appname)
 
 
