@@ -6,9 +6,9 @@
 
 #pragma once
 
+#include "vt-parser.h"
 #include "graphics.h"
 #include "monotonic.h"
-#define MAX_PARAMS 256
 
 typedef enum ScrollTypes { SCROLL_LINE = -999999, SCROLL_PAGE, SCROLL_FULL } ScrollType;
 
@@ -17,7 +17,6 @@ typedef struct {
          mBRACKETED_PASTE, mFOCUS_TRACKING, mDECSACE, mHANDLE_TERMIOS_SIGNALS;
     MouseTrackingMode mouse_tracking_mode;
     MouseTrackingProtocol mouse_tracking_protocol;
-    bool eight_bit_controls;  // S8C1T
 } ScreenModes;
 
 typedef struct {
@@ -57,12 +56,14 @@ typedef struct {
 
 #define SAVEPOINTS_SZ 256
 
+typedef struct CharsetState {
+    uint32_t *zero, *one, *current, current_num;
+} CharsetState;
+
 typedef struct {
-    uint32_t utf8_state, utf8_codepoint, *g0_charset, *g1_charset;
-    unsigned int current_charset;
-    bool use_latin1;
     Cursor cursor;
     bool mDECOM, mDECAWM, mDECSCNM;
+    CharsetState charset;
     bool is_valid;
 } Savepoint;
 
@@ -87,20 +88,17 @@ typedef struct {
 typedef struct {
     PyObject_HEAD
 
-    unsigned int columns, lines, margin_top, margin_bottom, charset, scrolled_by;
+    unsigned int columns, lines, margin_top, margin_bottom, scrolled_by;
     double pending_scroll_pixels_x, pending_scroll_pixels_y;
     CellPixelSize cell_size;
     OverlayLine overlay_line;
     id_type window_id;
-    uint32_t utf8_codepoint, *g0_charset, *g1_charset, *g_charset;
-    UTF8State utf8_state;
-    unsigned int current_charset;
     Selections selections, url_ranges;
     struct {
         unsigned int cursor_x, cursor_y, scrolled_by;
         index_type lines, columns;
     } last_rendered;
-    bool use_latin1, is_dirty, scroll_changed, reload_all_gpu_data;
+    bool is_dirty, scroll_changed, reload_all_gpu_data;
     Cursor *cursor;
     Savepoint main_savepoint, alt_savepoint;
     PyObject *callbacks, *test_child;
@@ -113,23 +111,12 @@ typedef struct {
     ColorProfile *color_profile;
     monotonic_t start_visual_bell_at;
 
-    uint32_t parser_buf[PARSER_BUF_SZ];
-    unsigned int parser_state, parser_text_start, parser_buf_pos;
-    bool parser_has_pending_text;
-    uint8_t read_buf[READ_BUF_SZ], *write_buf;
-    monotonic_t new_input_at;
-    size_t read_buf_sz, write_buf_sz, write_buf_used;
-    pthread_mutex_t read_buf_lock, write_buf_lock;
+    uint8_t *write_buf;
+    size_t write_buf_sz, write_buf_used;
+    pthread_mutex_t write_buf_lock;
 
     CursorRenderInfo cursor_render_info;
-    unsigned int render_unfocused_cursor;
 
-    struct {
-        size_t capacity, used;
-        uint8_t *buf;
-        monotonic_t activated_at, wait_time;
-        unsigned stop_escape_code_type;
-    } pending_mode;
     DisableLigature disable_ligatures;
     PyObject *marker;
     bool has_focus;
@@ -145,6 +132,7 @@ typedef struct {
     union {
         struct {
             unsigned int redraws_prompts_at_all: 1;
+            unsigned int uses_special_keys_for_cursor_movement: 1;
         };
         unsigned int val;
     } prompt_settings;
@@ -167,11 +155,21 @@ typedef struct {
     struct {
         uint8_t stack[16], count;
     } main_pointer_shape_stack, alternate_pointer_shape_stack;
+    Parser *vt_parser;
+    struct {
+        monotonic_t expires_at;
+        Cursor cursor;
+        ColorProfile color_profile;
+        bool inverted, cell_data_updated, cursor_visible;
+        unsigned int scrolled_by;
+        LineBuf *linebuf;
+        GraphicsManager *grman;
+        Selections selections, url_ranges;
+    } paused_rendering;
+    CharsetState charset;
 } Screen;
 
 
-void parse_worker(Screen *screen, PyObject *dump_callback, monotonic_t now);
-void parse_worker_dump(Screen *screen, PyObject *dump_callback, monotonic_t now);
 void screen_align(Screen*);
 void screen_restore_cursor(Screen *);
 void screen_save_cursor(Screen *);
@@ -184,7 +182,7 @@ void screen_cursor_position(Screen*, unsigned int, unsigned int);
 void screen_cursor_back(Screen *self, unsigned int count/*=1*/, int move_direction/*=-1*/);
 void screen_erase_in_line(Screen *, unsigned int, bool);
 void screen_erase_in_display(Screen *, unsigned int, bool);
-void screen_draw(Screen *screen, uint32_t codepoint, bool);
+void screen_draw_text(Screen *self, const uint32_t *chars, size_t num_chars);
 void screen_ensure_bounds(Screen *self, bool use_margins, bool cursor_was_within_margins);
 void screen_toggle_screen_buffer(Screen *self, bool, bool);
 void screen_normal_keypad_mode(Screen *self);
@@ -221,28 +219,22 @@ void screen_repeat_character(Screen *self, unsigned int count);
 void screen_delete_characters(Screen *self, unsigned int count);
 void screen_erase_characters(Screen *self, unsigned int count);
 void screen_set_margins(Screen *self, unsigned int top, unsigned int bottom);
-void screen_change_charset(Screen *, uint32_t to);
-void screen_handle_cmd(Screen *, PyObject *cmd);
 void screen_push_colors(Screen *, unsigned int);
 void screen_pop_colors(Screen *, unsigned int);
 void screen_report_color_stack(Screen *);
 void screen_handle_kitty_dcs(Screen *, const char *callback_name, PyObject *cmd);
-void screen_designate_charset(Screen *, uint32_t which, uint32_t as);
-void screen_use_latin1(Screen *, bool);
 void set_title(Screen *self, PyObject*);
 void desktop_notify(Screen *self, unsigned int, PyObject*);
 void set_icon(Screen *self, PyObject*);
 void set_dynamic_color(Screen *self, unsigned int code, PyObject*);
 void clipboard_control(Screen *self, int code, PyObject*);
-void shell_prompt_marking(Screen *self, PyObject*);
+void shell_prompt_marking(Screen *self, char *buf);
 void file_transmission(Screen *self, PyObject*);
 void set_color_table_color(Screen *self, unsigned int code, PyObject*);
-void process_cwd_notification(Screen *self, unsigned int code, PyObject*);
-uint32_t* translation_table(uint32_t which);
-void screen_request_capabilities(Screen *, char, PyObject *);
-void screen_set_8bit_controls(Screen *, bool);
+void process_cwd_notification(Screen *self, unsigned int code, const char*, size_t);
+void screen_request_capabilities(Screen *, char, const char *);
 void report_device_attributes(Screen *self, unsigned int UNUSED mode, char start_modifier);
-void select_graphic_rendition(Screen *self, int *params, unsigned int count, Region*);
+void select_graphic_rendition(Screen *self, int *params, unsigned int count, bool is_group, Region *r);
 void report_device_status(Screen *self, unsigned int which, bool UNUSED);
 void report_mode_status(Screen *self, unsigned int which, bool);
 void screen_apply_selection(Screen *self, void *address, size_t size);
@@ -261,7 +253,6 @@ void screen_update_selection(Screen *self, index_type x, index_type y, bool in_l
 bool screen_history_scroll(Screen *self, int amt, bool upwards);
 PyObject* as_text_history_buf(HistoryBuf *self, PyObject *args, ANSIBuf *output);
 Line* screen_visual_line(Screen *self, index_type y);
-unsigned long screen_current_char_width(Screen *self);
 void screen_mark_url(Screen *self, index_type start_x, index_type start_y, index_type end_x, index_type end_y);
 void set_active_hyperlink(Screen*, char*, char*);
 hyperlink_id_type screen_mark_hyperlink(Screen*, index_type, index_type);
@@ -285,6 +276,10 @@ int screen_cursor_at_a_shell_prompt(const Screen *);
 bool screen_fake_move_cursor_to_position(Screen *, index_type x, index_type y);
 bool screen_send_signal_for_key(Screen *, char key);
 bool get_line_edge_colors(Screen *self, color_type *left, color_type *right);
+bool parse_sgr(Screen *screen, const uint8_t *buf, unsigned int num, const char *report_name, bool is_deccara);
+bool screen_pause_rendering(Screen *self, bool pause, int for_in_ms);
+void screen_check_pause_rendering(Screen *self, monotonic_t now);
+void screen_designate_charset(Screen *self, uint32_t which, uint32_t as);
 #define DECLARE_CH_SCREEN_HANDLER(name) void screen_##name(Screen *screen);
 DECLARE_CH_SCREEN_HANDLER(bell)
 DECLARE_CH_SCREEN_HANDLER(backspace)
