@@ -63,7 +63,7 @@ GlobalState global_state = {{0}};
 
 static double
 dpi_for_os_window(OSWindow *os_window) {
-    double dpi = (os_window->logical_dpi_x + os_window->logical_dpi_y) / 2.;
+    double dpi = (os_window->fonts_data->logical_dpi_x + os_window->fonts_data->logical_dpi_y) / 2.;
     if (dpi == 0) dpi = (global_state.default_dpi.x + global_state.default_dpi.y) / 2.;
     return dpi;
 }
@@ -224,7 +224,6 @@ add_os_window(void) {
         }
     }
 
-    ans->font_sz_in_pts = OPT(font_size);
     END_WITH_OS_WINDOW_REFS
     return ans;
 }
@@ -751,7 +750,7 @@ PYWRAP1(set_ignore_os_keyboard_processing) {
 }
 
 static void
-init_screen_render_data(OSWindow *osw, const WindowGeometry *g, ScreenRenderData *d) {
+init_window_render_data(OSWindow *osw, const WindowGeometry *g, WindowRenderData *d) {
     d->dx = gl_size(osw->fonts_data->cell_width, osw->viewport_width);
     d->dy = gl_size(osw->fonts_data->cell_height, osw->viewport_height);
     d->xstart = gl_pos_x(g->left, osw->viewport_width);
@@ -759,14 +758,14 @@ init_screen_render_data(OSWindow *osw, const WindowGeometry *g, ScreenRenderData
 }
 
 PYWRAP1(set_tab_bar_render_data) {
-    ScreenRenderData d = {0};
+    WindowRenderData d = {0};
     WindowGeometry g = {0};
     id_type os_window_id;
     PA("KOIIII", &os_window_id, &d.screen, &g.left, &g.top, &g.right, &g.bottom);
     WITH_OS_WINDOW(os_window_id)
         Py_CLEAR(os_window->tab_bar_render_data.screen);
         d.vao_idx = os_window->tab_bar_render_data.vao_idx;
-        init_screen_render_data(os_window, &g, &d);
+        init_window_render_data(os_window, &g, &d);
         os_window->tab_bar_render_data = d;
         Py_INCREF(os_window->tab_bar_render_data.screen);
     END_WITH_OS_WINDOW
@@ -872,10 +871,22 @@ PYWRAP1(run_with_activation_token) {
         OSWindow *os_window = global_state.os_windows + o;
         if (os_window->is_focused) {
             run_with_activation_token_in_os_window(os_window, args);
-            break;
+            Py_RETURN_TRUE;
         }
     }
-    Py_RETURN_NONE;
+    id_type os_window_id = last_focused_os_window_id();
+    if (!os_window_id) {
+        if (!global_state.num_os_windows) Py_RETURN_FALSE;
+        os_window_id = global_state.os_windows[0].id;
+    }
+    for (size_t o = 0; o < global_state.num_os_windows; o++) {
+        OSWindow *os_window = global_state.os_windows + o;
+        if (os_window->id == os_window_id) {
+            run_with_activation_token_in_os_window(os_window, args);
+            Py_RETURN_TRUE;
+        }
+    }
+    Py_RETURN_FALSE;
 }
 
 PYWRAP1(set_os_window_chrome) {
@@ -930,14 +941,14 @@ PYWRAP1(set_window_render_data) {
 #define A(name) &(d.name)
 #define B(name) &(g.name)
     id_type os_window_id, tab_id, window_id;
-    ScreenRenderData d = {0};
+    WindowRenderData d = {0};
     WindowGeometry g = {0};
     PA("KKKOIIII", &os_window_id, &tab_id, &window_id, A(screen), B(left), B(top), B(right), B(bottom));
 
     WITH_WINDOW(os_window_id, tab_id, window_id);
         Py_CLEAR(window->render_data.screen);
         d.vao_idx = window->render_data.vao_idx;
-        init_screen_render_data(osw, &g, &d);
+        init_window_render_data(osw, &g, &d);
         window->render_data = d;
         window->geometry = g;
         Py_INCREF(window->render_data.screen);
@@ -1020,10 +1031,10 @@ PYWRAP1(os_window_font_size) {
     double new_sz = -1;
     PA("K|dp", &os_window_id, &new_sz, &force);
     WITH_OS_WINDOW(os_window_id)
-        if (new_sz > 0 && (force || new_sz != os_window->font_sz_in_pts)) {
-            os_window->font_sz_in_pts = new_sz;
-            os_window->fonts_data = NULL;
-            os_window->fonts_data = load_fonts_data(os_window->font_sz_in_pts, os_window->logical_dpi_x, os_window->logical_dpi_y);
+        if (new_sz > 0 && (force || new_sz != os_window->fonts_data->font_sz_in_pts)) {
+            double xdpi, ydpi; float xscale, yscale;
+            get_os_window_content_scale(os_window, &xdpi, &ydpi, &xscale, &yscale);
+            os_window->fonts_data = load_fonts_data(new_sz, xdpi, ydpi);
             send_prerendered_sprites_for_window(os_window);
             resize_screen(os_window, os_window->tab_bar_render_data.screen, false);
             for (size_t ti = 0; ti < os_window->num_tabs; ti++) {
@@ -1037,7 +1048,7 @@ PYWRAP1(os_window_font_size) {
             // On Wayland with CSD title needs to be re-rendered in a different font size
             if (os_window->window_title && global_state.is_wayland) set_os_window_title(os_window, NULL);
         }
-        return Py_BuildValue("d", os_window->font_sz_in_pts);
+        return Py_BuildValue("d", os_window->fonts_data->font_sz_in_pts);
     END_WITH_OS_WINDOW
     return Py_BuildValue("d", 0.0);
 }
