@@ -13,6 +13,7 @@ from kitty.fast_data_types import (
     LineBuf,
     expand_ansi_c_escapes,
     parse_input_from_terminal,
+    replace_c0_codes_except_nl_space_tab,
     strip_csi,
     truncate_point_for_length,
     wcswidth,
@@ -37,6 +38,18 @@ def create_lbuf(*lines):
 
 class TestDataTypes(BaseTest):
 
+
+    def test_replace_c0_codes(self):
+        def t(x: str, expected: str):
+            q = replace_c0_codes_except_nl_space_tab(x)
+            self.ae(expected, q)
+            q = replace_c0_codes_except_nl_space_tab(x.encode('utf-8'))
+            self.ae(expected.encode('utf-8'), q)
+        t('abc', 'abc')
+        t('a\0\x01b\x03\x04\t\rc', 'a\u2400\u2401b\u2403\u2404\t\u240dc')
+        t('a\0\x01😸\x03\x04\t\rc', 'a\u2400\u2401😸\u2403\u2404\t\u240dc')
+        t('a\nb\tc d', 'a\nb\tc d')
+
     def test_to_color(self):
         for x in 'xxx #12 #1234 rgb:a/b'.split():
             self.assertIsNone(to_color(x))
@@ -616,7 +629,7 @@ class TestDataTypes(BaseTest):
 
     def test_shlex_split(self):
         for bad in (
-            'abc\\', '\\', "'abc", "'", '"', 'asd' + '\\',
+            'abc\\', '\\', "'abc", "'", '"', 'asd' + '\\', r'"a\"', '"a\\',
         ):
             with self.assertRaises(ValueError, msg=f'Failed to raise exception for {bad!r}'):
                 tuple(shlex_split_with_positions(bad))
@@ -627,6 +640,24 @@ class TestDataTypes(BaseTest):
             r'''x'y"\z'1''': ((0, 'xy"\\z1'),),
             r'\abc\ d': ((0, 'abc d'),),
             '': (), '   ': (), ' \tabc\n\t\r ': ((2, 'abc'),),
+            "$'ab'": ((0, '$ab'),),
         }.items():
             actual = tuple(shlex_split_with_positions(q))
             self.ae(expected, actual, f'Failed for text: {q!r}')
+
+        for q, expected in {
+            "$'ab'": ((0, 'ab'),),
+            "1$'ab'": ((0, '1ab'),),
+            '''"1$'ab'"''': ((0, "1$'ab'"),),
+            r"$'a\123b'": ((0, 'a\123b'),),
+            r"$'a\1b'": ((0, 'a\001b'),),
+            r"$'a\12b'": ((0, 'a\012b'),),
+            r"$'a\db'": ((0, 'adb'),),
+            r"$'a\x1bb'": ((0, 'a\x1bb'),),
+            r"$'\u123z'": ((0, '\u0123z'),),
+            r"$'\U0001F1E8'": ((0, '\U0001F1E8'),),
+            r"$'\U1F1E8'": ((0, '\U0001F1E8'),),
+            r"$'a\U1F1E8'b": ((0, 'a\U0001F1E8b'),),
+        }.items():
+            actual = tuple(shlex_split_with_positions(q, True))
+            self.ae(expected, actual, f'Failed for text: {q!r}')
